function colout = colScaler(pic, xout, coeff)

% Number of phases between each input pixel
nphases = size(coeff,1);

% Number of filter taps
ntaps = size(coeff,2);

% Coefficients for each phase equals 128 in sum
csum = 128;

    % Find the closest "global" phase (i.e. phasenumer doesn't wrap)
    % Say globalphase=0 is at the first input pixel (i.e. offset 0.5)
    globalphase = floor((xout)*nphases);

    % Find closest phase of output pixel  (1, 2, ... nphases)
    phase = mod(globalphase, nphases) + 1 ;

    column = zeros(size(pic,1), 1);

    % The input pixel index corresponding to the first filter tap is:
    xin = floor(globalphase/nphases) - (floor(ntaps/2)-1) + 1;

    % Parse each tap individually
    for tap=1:ntaps
        % Accumulate output column, filtering one tap at a time
        % Limit range of xin to actual input picture pixel range.
        % Will also replicate input pixels at the picture borders.
        if(xin >= 1 && xin <= size(pic,2)) % causes locations outside the 
            % image to be black instead of the color of the border
            column = column + coeff(phase, tap) * cast(pic(:, xin), 'double');
        end
        % Next tap applies to next pixel
        xin = xin + 1;
    end

    colout = limit(round(column / csum), 0, 255);
end