library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.pscaler_local_package.all;

entity pscaler_filter is

  generic (
    G_PIXEL_DEPTH   : integer := 8;  -- Number of bits representing one pixel value
    G_HMAXOUT_WIDTH : integer := 11;  -- Number of bits in the pixel index variable. Max number of pixels is 2^G_INDEX_WIDTH.
    G_VMAXOUT_WIDTH : integer := 11);  -- number of bits representing the decimal value of pixel position
  port (
    clk       : in  std_logic;
    arst      : in  std_logic;
    ready     : in  std_logic;
    ready_out : out std_logic;

    -- frame buffer interface
    frame_buffer_xdat         : out std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_ZERO_IND);
    frame_buffer_ydat         : out std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_ZERO_IND);
    frame_buffer_request_data : out std_logic;
    frame_buffer_value        : in  std_logic_vector(G_PIXEL_DEPTH-1 downto 0);

    xPos           : in  std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
    yPos           : in  std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
    position_valid : in  std_logic;
    outofbounds    : in  std_logic;
    filter_value   : out std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
    datavalid_out  : out std_logic);

end pscaler_filter;

architecture nearestNeighbourRTL of pscaler_filter is

  signal dvo_fifo    : std_logic_vector(2 downto 0);

  constant outofboundscolor : std_logic_vector(filter_value'range) := (others => '0');

begin

  -- because every pixel is set to start in (0.5,0.5) in the mapper, the
  -- nn-filter should only floor the values (round them down to nearest integer):
  frame_buffer_xdat <= std_logic_vector(xPos(frame_buffer_xdat'range));
  frame_buffer_ydat <= std_logic_vector(yPos(frame_buffer_ydat'range));

  out_gen : process (clk)
  begin
    
    if rising_edge(clk) then

      if outofbounds = '1' then         --overflow
        filter_value <= outofboundscolor;
      else
        filter_value <= frame_buffer_value;
      end if;

      dvo_fifo(dvo_fifo'high)            <= position_valid;
      dvo_fifo(dvo_fifo'high-1 downto 0) <= dvo_fifo(dvo_fifo'high downto 1);
    end if;

  end process out_gen;

  datavalid_out <= dvo_fifo(0);         -- data from RAM is delayed with 3
                                        -- clock cycles. 

end nearestNeighbourRTL;
