	 library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


library std;
use std.textio.all;

--library PerspectiveScaler;
--use PerspectiveScaler.pscaler_local_package.all;
use work.pscaler_local_package.all;

entity pscaler_tb is

end pscaler_tb;

architecture tb of pscaler_tb is

-------------------------------------------------------------------------------
-- DUT generics
-------------------------------------------------------------------------------
  constant G_HMAXOUT_WIDTH            : integer := 7;
  constant G_VMAXOUT_WIDTH            : integer := 7;
  constant G_PERSPECTIVE_FACTOR_WIDTH : integer := 8;
  constant G_DECIMAL_PRECISION        : integer := 5;
  constant G_PIXEL_DEPTH              : integer := 8;

-------------------------------------------------------------------------------
-- DUT signals
-------------------------------------------------------------------------------
  signal clk       : std_logic := '0';
  signal arst      : std_logic;
  signal ready     : std_logic;
  signal ready_out : std_logic;

  signal enable : std_logic;

  signal image_start  : std_logic;
  signal vscale_start : std_logic;
  signal data_in      : std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
  signal datavalid_in : std_logic;

  signal input_hsize    : std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
  signal Px             : std_logic_vector(G_PERSPECTIVE_FACTOR_WIDTH-1 downto 0);
  signal input_vsize    : std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
  signal Py             : std_logic_vector(G_PERSPECTIVE_FACTOR_WIDTH-1 downto 0);
  signal data_out       : std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
  signal datavalid_out  : std_logic;
  signal Py_slope_start : std_logic_vector(7 downto 0);

-------------------------------------------------------------------------------
-- other signals
-------------------------------------------------------------------------------

  signal current_row    : integer := 0;
  signal current_col    : integer := 0;
  signal input_finished : boolean := false;

  signal start_data_gen : std_logic := '0';
  
begin  -- tb

-------------------------------------------------------------------------------
-- clk generation
-------------------------------------------------------------------------------

  p_clk_gen : process
  begin  -- process p_clk_gen
    wait for 10 ns;
    clk <= not clk;
  end process p_clk_gen;

  reset : process
  begin  -- process reset
    wait until rising_edge(clk);
    arst           <= '0';
    wait until rising_edge(clk);
    arst           <= '1';
    wait until rising_edge(clk);
    arst           <= '0';
    start_data_gen <= '1';
    wait;
  end process reset;

-------------------------------------------------------------------------------
-- Input generation
-------------------------------------------------------------------------------

-- 16x16 picture:
--  Px             <= "00001101";
--  Py             <= "01001101";
--  Py_slope_start <= "00010010";

-- 128x128 picture:
  Px             <= "00010010";
  Py             <= "00001001";
  Py_slope_start <= "00010011";
  
  enable       <= '1';
  input_hsize  <= (others => '1');
  input_vsize  <= (others => '1');
  ready        <= '1';
  vscale_start <= '0';                  -- trenger jeg denne?


--  p_input_gen : process(clk)
--  begin  -- process p_input_gen
--    -- default:
--    if rising_edge(clk) then
--      image_start  <= '0';
--      datavalid_in <= '0';
--      if (not input_finished) and (start_data_gen = '1') then
--        datavalid_in <= ready_out;
--        if rising_edge(clk) then
--          if ready_out = '1' then
--            if current_row = 0 and current_col = 0 then
--              image_start <= '1';
--            end if;
--            -- generate input
--            data_in     <= std_logic_vector(to_unsigned(current_row+current_col*2**G_HMAXOUT_WIDTH, G_PIXEL_DEPTH));
--            current_row <= current_row+1;
--            if current_row = 2**G_HMAXOUT_WIDTH-1 then
--              current_row <= 0;
--              current_col <= current_col+1;
--              if current_col = 2**G_VMAXOUT_WIDTH-1 then
--                current_col    <= 0;
--                input_finished <= true;
--              end if;
--            end if;
--          end if;
--        end if;
--      end if;
--    end if;
--  end process p_input_gen;

-------------------------------------------------------------------------------
-- File IO
-------------------------------------------------------------------------------

  file_read : process
    variable linenumber    : integer := 1;
    file infile            : text is in "C:\Thesis\Code\matlabmodell\test_input.txt";
    variable inline        : line;
    variable dataread_int  : integer;
    variable image_started : boolean := false;
  begin  -- process file_io

    if start_data_gen = '0' then
      wait until start_data_gen = '1';
    end if;

    wait until rising_edge(clk);
    data_in      <= (others => '0');
    datavalid_in <= '0';
    image_start  <= '0';

    if (ready_out = '1' and not endfile(infile)) then  --checking the "END OF FILE" is not reached.

      readline(infile, inline);         --reading a line from the file.
      read(inline, dataread_int);

      if image_started = false then
        image_start   <= '1';
        image_started := true;
      end if;

      data_in      <= std_logic_vector(to_unsigned(dataread_int, 8));  --put the value available in variable in a signal.
      datavalid_in <= '1';
     
    end if;
    
  end process file_read;

  file_write : process

    file outfile     : text is out "C:\Thesis\Code\matlabmodell\test_output.txt";  --declare output file
    variable outline : line;            --line number declaration  
  begin
    wait until rising_edge(clk);
    if datavalid_out = '1' then
--write(linenumber,value(real type),justified(side),field(width),digits(natural));
        write(outline, to_integer(unsigned(data_out)));
-- write line to external file.
        writeline(outfile, outline);
    end if;
  end process file_write;


-------------------------------------------------------------------------------
-- DUT initialization
-------------------------------------------------------------------------------
  DUT : pscaler_top
    generic map (
      G_HMAXOUT_WIDTH => G_HMAXOUT_WIDTH,
      G_VMAXOUT_WIDTH => G_VMAXOUT_WIDTH,
      G_PIXEL_DEPTH   => G_PIXEL_DEPTH)
    port map (
      clk            => clk,
      arst           => arst,
      ready          => ready,
      ready_out      => ready_out,
      image_start    => image_start,
      data_in        => data_in,
      datavalid_in   => datavalid_in,
      input_hsize    => input_hsize,
      Px             => Px,
      input_vsize    => input_vsize,
      Py             => Py,
      Py_slope_start => Py_slope_start,
      data_out       => data_out,
      datavalid_out  => datavalid_out);

end tb;
